/*		

November !
	
Variation autour d'un Automate Cellulaire,
2D Asynchrone. 

Version / 10 septembre 2010.

*/

/*


	Moore			 Diamond			   Square_head			 Rectangle

					  | W |					M | W | N		     M | W | N
  A | B | C			A | B | C				A | B | C			 A | B | C
  D |  | E		X | D |  | E | Y		X | D |  | E | Y		 D |  | E
  F | G | H 		F | G | H				F | G | H			 F | G | H
					  | Z |					O | Z | P			 O | Z | P
					  
		Cross				   

  I |           | J
    | A |   | C |						A |   | C
		|  |					Hpat :	D |  | E
    | F |   | H |						F |   | H
  K |           | L


*/

// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

#include "jit.common.h"							
#include "ext_obex.h"
#include "jpatcher_api.h"
#include "stdlib.h"
#include "math.h"	
#include "time.h"	
#include "ext_systhread.h"

// -------------------------------------------------------------------------------------------------------------

#define ALFA						(x->size_row - 1)
#define BRAVO						(x->size_row)
#define CHARLIE						(x->size_row + 1)
#define DELTA						(-1)
#define ECHO						(1)
#define FOXTROT						(-x->size_row - 1)
#define GULF						(-x->size_row)
#define HOTEL						(-x->size_row + 1)
#define WHISKEY						(2 * x->size_row)
#define XRAY						(-2)
#define YANKEE						(2)
#define ZULU						(-2 * x->size_row)
#define MIKE						((2 * x->size_row) - 1)
#define NOVEMBER					((2 * x->size_row) + 1)
#define OSCAR						((-2 * x->size_row) - 1)
#define PAPA						((-2 * x->size_row) + 1)
#define	INDIA						((2 * x->size_row) - 2)
#define	JULIET						((2 * x->size_row) + 2)
#define KILO						((-2 * x->size_row) - 2)
#define LIMA						((-2 * x->size_row) + 2)

#define H_PATTERN_NUMBER			4
#define MAX_SIZE_PATTERN			17
#define	SAFE_LOOP					20

#define DEF_PATCHER_TO_FIND			"totoestunamidejojolevoisindelola"
#define DEF_VARNAME_TO_FIND			"lolaestlavoisinedejojounamidetoto"
#define MATRIX_TO_FIND_PLANECOUNT	8     
#define MAX_NUMBER_NOTES			255                                                      
#define MAX_POS						5759        

#define MAX_MAT_CONTENT				500
#define MAX_NOVEMBER_CONTENT		2000
#define MAX_SIZE_ARRAY				368640    

#define DEF_SPRAY					0
#define DEF_ITER					1
#define DEF_PATTERN					"rectangle"
#define	DEF_ENUMINDEX				3
#define	DEF_GRID					"4n"
#define DEF_RULES					"l'aspiraspore"
#define DEF_BIRTH_LOW				1
#define	DEF_BIRTH_WIDE				0
#define	DEF_DEATH_LOW				0
#define	DEF_DEATH_WIDE				1
#define	DEF_STEP					24
#define	DEF_SIZE_ROW				240         
#define DEF_SIZE_ARRAY				30720 

/*
	MAX_POS			: -badaboum.js- // (SEQ_BAR_LONG * 4 * STEP) - 1 
	DEF_STEP		: -badaboum.js- // STEP
*/

// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

typedef struct _mat_head {
	long		size ;
	long		index ;
	long		*content ;
	t_object	*jptr ;
	t_object	*optr ;
	} t_mat_head ;
	
typedef struct _note_head {
	long	link ;
	t_atom	*note ;
	} t_note_head ;

typedef struct _november {
	t_object			ob ;
	t_object			*mypatcher ;			
	t_object			*mypatcherview ;
	t_symbol			*catch_name ;
	t_atom				grid ;
	t_atom				neighbours ;
	t_atom				rules ;
	t_linklist			*badaboum ;
	t_note_head			**board ;
	long				*content ;
	long				count ;
	long				index ;
	long				iter ;
	long				temp ;
	long				step ;
	long				size_row ;
	long				size_array ;
	long				spray ;
	long				*Hpat_A ;
	long				*Hpat_B ;
	long				*Hpat_C ;
	long				*Hpat_D ;
	long				*cluster ;
	long				*moore ;
	long				*diamond ;
	long				*square_head ;
	long				*rectangle ;
	long				*cross ;
	long				*neighbourhood ;
	char				enumindex ;
	char				dirty ;
	char				birth_low ;
	char				birth_wide ;
	char				death_low ;
	char				death_wide ;
	void				*m_out ;
	t_systhread_mutex	mymutex ;
	} t_november ;
	
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------	
	
void		*november_new				(t_symbol *s, long argc, t_atom *argv) ;
t_max_err	november_setgrid			(t_november *x, t_object *attr, long argc, t_atom *argv) ;
t_max_err	november_getgrid			(t_november *x, t_object *attr, long *argc, t_atom **argv) ;
t_max_err	november_setrules			(t_november *x, t_object *attr, long argc, t_atom *argv) ;
t_max_err	november_getrules			(t_november *x, t_object *attr, long *argc, t_atom **argv) ;
t_max_err	november_setneighbours		(t_november *x, t_object *attr, long argc, t_atom *argv) ;
t_max_err	november_getneighbours		(t_november *x, t_object *attr, long *argc, t_atom **argv) ;
void		november_free				(t_november *x) ;
void		november_assist				(t_november *x, void *b, long m, long a, char *s) ;
void		november_attach				(t_november *x) ;
void		november_notify				(t_november *x, t_symbol *s, t_symbol *msg, void *sender, void *data) ;

void		november_bang				(t_november *x) ;
void		november_shake				(t_november *x) ;
void		november_swap				(t_november *x) ;
void		november_values				(t_november *x, t_symbol *s, long ac, t_atom *av) ;

void		pickup_matrix				(t_november *x) ;
long		find_matrix					(t_november *x, t_object *ptr_b) ;
void		free_badaboum_content		(t_november *x)	;
void		clear_badaboum_content		(t_november *x)	;
void		pickup_note					(t_november *x) ;
void		free_board					(t_november *x) ;
void		fill_matrix					(t_november *x) ;

void			process_automata		(t_november *x) ;
long			neighbourhood_death		(t_november *x, long n) ;
t_note_head		*neighbourhood_birth	(t_november *x, long n) ;

void			process_shake			(t_november *x) ;
void			process_swap			(t_november *x) ;

void			init_pattern			(t_november *x) ;
void			set_neighbourhood		(t_november *x) ;
long			*getrandom_pattern		(t_november *x) ;
long			getrandom_content		(t_november *x) ;

// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

static t_class	*november_class ;

static t_symbol	*my_jit_matrix	= NULL ;
static t_symbol	*my_getcell		= NULL ;
static t_symbol	*my_setcell		= NULL ;

// -------------------------------------------------------------------------------------------------------------

int main (void)
	{	
		t_class *c = NULL ;
		
		my_jit_matrix	= gensym ("jit.matrix") ;
		my_getcell		= gensym ("getcell") ;
		my_setcell		= gensym ("setcell") ;
		
		common_symbols_init () ;
		
		c = class_new ("november", (method)november_new, (method)november_free, (long)sizeof(t_november), 0L, A_GIMME, 0) ;
		
		class_addmethod (c, (method)november_assist,	"assist", A_CANT, 0) ;
		class_addmethod (c, (method)november_notify,	"notify", A_CANT, 0) ; 
		class_addmethod (c, (method)november_bang,		"bang", 0) ;
		class_addmethod (c, (method)november_shake,		"shake", 0) ;
		class_addmethod (c, (method)november_swap,		"swap", 0) ;
		class_addmethod (c, (method)november_values,	"values", A_GIMME, 0) ;
		
		class_register (CLASS_BOX, c) ; 
		
		CLASS_ATTR_ATOM			(c, "grid",				ATTR_FLAGS_NONE, t_november, grid) ;
		CLASS_ATTR_ENUM			(c, "grid",				ATTR_FLAGS_NONE, "1n 2n 4n 8n 16n 32n 1nt 2nt 4nt 8nt 16nt 32nt") ;
		CLASS_ATTR_ACCESSORS	(c, "grid",				november_getgrid, november_setgrid) ;
		CLASS_ATTR_SAVE			(c, "grid",				ATTR_FLAGS_NONE) ;
		CLASS_ATTR_LABEL		(c, "grid",				ATTR_FLAGS_NONE, "Grid") ;
		
		CLASS_ATTR_ATOM			(c, "neighbourhood",	ATTR_FLAGS_NONE, t_november, neighbours) ;
		CLASS_ATTR_ENUM			(c, "neighbourhood",	ATTR_FLAGS_NONE, "moore diamond square_head rectangle cross") ;
		CLASS_ATTR_ACCESSORS	(c, "neighbourhood",	november_getneighbours, november_setneighbours) ;
		CLASS_ATTR_SAVE			(c, "neighbourhood",	ATTR_FLAGS_NONE) ;
		CLASS_ATTR_LABEL		(c, "neighbourhood",	ATTR_FLAGS_NONE, "Neighbourhood") ;
		
		CLASS_ATTR_ATOM			(c, "rules",			ATTR_FLAGS_NONE, t_november, rules) ;
		CLASS_ATTR_ENUM			(c, "rules",			ATTR_FLAGS_NONE, "l'aspiraspore boogie_woogie gameoflife gol- gol+") ;
		CLASS_ATTR_ACCESSORS	(c, "rules",			november_getrules, november_setrules) ;
		CLASS_ATTR_SAVE			(c, "rules",			ATTR_FLAGS_NONE) ;
		CLASS_ATTR_LABEL		(c, "rules",			ATTR_FLAGS_NONE, "Rules") ;
		
		CLASS_ATTR_LONG			(c, "iterate",			ATTR_FLAGS_NONE, t_november, iter) ;
		CLASS_ATTR_FILTER_CLIP	(c, "iterate",			0, 100) ;
		CLASS_ATTR_SAVE			(c, "iterate",			ATTR_FLAGS_NONE) ;
		CLASS_ATTR_LABEL		(c, "iterate",			ATTR_FLAGS_NONE, "How Many Times ?") ;
		
		CLASS_ATTR_LONG			(c, "spray",			ATTR_FLAGS_NONE, t_november, spray) ;
		CLASS_ATTR_FILTER_CLIP	(c, "spray",			0, 1) ;
		CLASS_ATTR_SAVE			(c, "spray",			ATTR_FLAGS_NONE) ;
		CLASS_ATTR_STYLE_LABEL	(c, "spray",			ATTR_FLAGS_NONE, "onoff", "Cluster Spray") ;
		
		CLASS_ATTR_ORDER		(c, "iterate",			ATTR_FLAGS_NONE, "1") ;
		CLASS_ATTR_ORDER		(c, "grid",				ATTR_FLAGS_NONE, "2") ;
		CLASS_ATTR_ORDER		(c, "rules",			ATTR_FLAGS_NONE, "3") ;
		CLASS_ATTR_ORDER		(c, "neighbourhood",	ATTR_FLAGS_NONE, "4") ;
		CLASS_ATTR_ORDER		(c, "spray",			ATTR_FLAGS_NONE, "5") ;
		
		CLASS_METHOD_ATTR_PARSE (c, "values", "undocumented", _sym_long, ATTR_FLAGS_NONE, "1") ;
		
		november_class = c ;
		
		return MAX_ERR_NONE ;
	}

// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

void *november_new (t_symbol *s, long argc, t_atom *argv)
	{
		t_november *x = NULL ;
		
		if (x = (t_november *)object_alloc (november_class)) 
			{	
				if (x->board = (t_note_head **)sysmem_newptrclear (sizeof(t_note_head *) * MAX_SIZE_ARRAY))
					{
						if (x->content = (long *)sysmem_newptrclear (sizeof(long) * MAX_NOVEMBER_CONTENT))
							{
								x->dirty			= 1 ;
								x->index			= 0 ;
								x->count			= 0 ;
								x->spray			= DEF_SPRAY ;
								x->iter				= DEF_ITER ;
								x->birth_low		= DEF_BIRTH_LOW ;
								x->birth_wide		= DEF_BIRTH_WIDE ;
								x->death_low		= DEF_DEATH_LOW ;
								x->death_wide		= DEF_DEATH_WIDE ;
								x->temp				= DEF_STEP ;
								x->step				= DEF_STEP ;
								x->size_row			= DEF_SIZE_ROW ;
								x->size_array		= DEF_SIZE_ARRAY ;
								x->enumindex		= DEF_ENUMINDEX ;
								x->neighbourhood	= NULL ;
								
								x->catch_name		= gensym (DEF_PATCHER_TO_FIND) ;
								
								atom_setsym			(&x->grid, gensym (DEF_GRID)) ;
								atom_setsym			(&x->neighbours, gensym (DEF_PATTERN)) ;
								atom_setsym			(&x->rules, gensym (DEF_RULES)) ;
								
								x->badaboum			= linklist_new () ;
								linklist_flags		(x->badaboum, OBJ_FLAG_MEMORY) ;
								
								object_obex_lookup (x, gensym ("#P"), &x->mypatcher) ;
								
								attr_args_process (x, argc, argv) ;
								
								if (atom_gettype (argv) == A_SYM)
									{
										if (argc % 2)
											{
												x->catch_name = atom_getsym (argv) ;	
											}
									}
										
								defer_low (x, (method)november_attach, NULL, 0, NULL) ;		
								defer_low (x, (method)pickup_matrix, NULL, 0, NULL) ;	
								
								init_pattern (x) ;
								
								systhread_mutex_new (&x->mymutex, SYSTHREAD_MUTEX_NORMAL) ;
								srand (time(NULL)) ;
								x->m_out = intout ((t_object *)x) ;
							}
						else
							{
								sysmem_freeptr (x->board) ;
								object_free (x) ;
								x = NULL ;
							}
					}	
				else
					{
						object_free (x) ;
						x = NULL ;
					}
			}
				
		return x ;
	}

t_max_err november_setgrid (t_november *x, t_object *attr, long argc, t_atom *argv)
    {
		if (argc && argv)
			{
				t_symbol *sym = atom_getsym (argv) ;
				
				if (sym == gensym ("1n")) 
					{ x->temp = 96 ; atom_setsym (&x->grid, sym) ; }
				if (sym == gensym ("2n")) 
					{ x->temp = 48 ; atom_setsym (&x->grid, sym) ; }
				if (sym == gensym ("4n")) 
					{ x->temp = 24 ; atom_setsym (&x->grid, sym) ; }     // -badaboum.js- : STEP 
				if (sym == gensym ("8n")) 
					{ x->temp = 12 ; atom_setsym (&x->grid, sym) ; }
				if (sym == gensym ("16n")) 
					{ x->temp = 6 ; atom_setsym (&x->grid, sym) ; }
				if (sym == gensym ("32n")) 
					{ x->temp = 3 ; atom_setsym (&x->grid, sym) ; }
				if (sym == gensym ("1nt")) 
					{ x->temp = 64 ; atom_setsym (&x->grid, sym) ; }
				if (sym == gensym ("2nt")) 
					{ x->temp = 32 ; atom_setsym (&x->grid, sym) ; }
				if (sym == gensym ("4nt")) 
					{ x->temp = 16 ; atom_setsym (&x->grid, sym) ; }
				if (sym == gensym ("8nt")) 
					{ x->temp = 8 ; atom_setsym (&x->grid, sym) ; }
				if (sym == gensym ("16nt")) 
					{ x->temp = 4 ; atom_setsym (&x->grid, sym) ; }
				if (sym == gensym ("32nt")) 
					{ x->temp = 2 ; atom_setsym (&x->grid, sym) ; }
			}

        return MAX_ERR_NONE ;
    }
	
t_max_err november_getgrid (t_november *x, t_object *attr, long *argc, t_atom **argv)
    {
        char		alloc ;
		t_symbol	*sym = NULL ; 
 
		sym = atom_getsym (&x->grid) ;
		
		if (!((*argc) && (*argv)))
			{
				if (atom_alloc (argc, argv, &alloc))
					{
						return MAX_ERR_OUT_OF_MEM ;
					}
			}
				
        atom_setsym (*argv, sym) ;
		
        return MAX_ERR_NONE ;
    }

t_max_err november_setrules (t_november *x, t_object *attr, long argc, t_atom *argv)
    {
		if (argc && argv)
			{
				t_symbol *sym = atom_getsym (argv) ;
				
				if (sym == gensym ("l'aspiraspore")) 
					{
						x->birth_low = 1 ;
						x->birth_wide = 0 ;
						x->death_low = 0 ;
						x->death_wide = 1 ;
						
						atom_setsym (&x->rules, sym) ;
					}
					
				if (sym == gensym ("boogie_woogie")) 
					{
						x->birth_low = 2 ;
						x->birth_wide = 0 ;
						x->death_low = 0 ;
						x->death_wide = 1 ;
						
						atom_setsym (&x->rules, sym) ;
					}
				
				if (sym == gensym ("gameoflife")) 
					{
						x->birth_low = 3 ;
						x->birth_wide = 0 ;
						x->death_low = 2 ;
						x->death_wide = 1 ;
						
						atom_setsym (&x->rules, sym) ;
					}
				
				if (sym == gensym ("gol-")) 
					{
						x->birth_low = 3 ;
						x->birth_wide = 0 ;
						x->death_low = 2 ;
						x->death_wide = 0 ;
						
						atom_setsym (&x->rules, sym) ;
					}
				
				if (sym == gensym ("gol+")) 
					{
						x->birth_low = 3 ;
						x->birth_wide = 0 ;
						x->death_low = 2 ;
						x->death_wide = 2 ;
						
						atom_setsym (&x->rules, sym) ;
					}
			}

        return MAX_ERR_NONE ;
    }
	
t_max_err november_getrules (t_november *x, t_object *attr, long *argc, t_atom **argv)
    {
        char		alloc ;
		t_symbol	*sym = NULL ; 
 
		sym = atom_getsym (&x->rules) ;
		
		if (!((*argc) && (*argv)))
			{
				if (atom_alloc (argc, argv, &alloc))
					{
						return MAX_ERR_OUT_OF_MEM ;
					}
			}
			
        atom_setsym (*argv, sym) ;
		
        return MAX_ERR_NONE ;
    }

t_max_err november_setneighbours (t_november *x, t_object *attr, long argc, t_atom *argv)
    {
		if (argc && argv)
			{
				t_symbol *sym = atom_getsym (argv) ; 
				
				if (sym == gensym ("moore")) 
					{ x->enumindex = 0 ; atom_setsym (&x->neighbours, sym) ; }
				if (sym == gensym ("diamond")) 
					{ x->enumindex = 1 ; atom_setsym (&x->neighbours, sym) ; }
				if (sym == gensym ("square_head")) 
					{ x->enumindex = 2 ; atom_setsym (&x->neighbours, sym) ; }
				if (sym == gensym ("rectangle")) 
					{ x->enumindex = 3 ; atom_setsym (&x->neighbours, sym) ; }
				if (sym == gensym ("cross")) 
					{ x->enumindex = 4 ; atom_setsym (&x->neighbours, sym) ; }
			}

        return MAX_ERR_NONE ;
    }
	
t_max_err november_getneighbours (t_november *x, t_object *attr, long *argc, t_atom **argv)
    {
        char		alloc ;
		t_symbol	*sym = NULL ; 
 
		sym = atom_getsym (&x->neighbours) ;
		
		if (!((*argc) && (*argv)))
			{
				if (atom_alloc (argc, argv, &alloc))
					{
						return MAX_ERR_OUT_OF_MEM ;
					}
			}
			
        atom_setsym (*argv, sym) ;
		
        return MAX_ERR_NONE ;
    }
	
void november_free (t_november *x)
	{ 
		if (x->mypatcherview) 
			{
				object_detach_byptr (x, x->mypatcherview) ;
			}
			
		sysmem_freeptr (x->content) ;
		sysmem_freeptr (x->board) ;
		
		free_badaboum_content (x) ;
		object_free (x->badaboum) ;
		
		sysmem_freeptr (x->Hpat_A) ;
		sysmem_freeptr (x->Hpat_B) ;
		sysmem_freeptr (x->Hpat_C) ;
		sysmem_freeptr (x->Hpat_D) ;
		sysmem_freeptr (x->cluster) ;
		sysmem_freeptr (x->moore) ;
		sysmem_freeptr (x->diamond) ;
		sysmem_freeptr (x->square_head) ;
		sysmem_freeptr (x->rectangle) ;
		sysmem_freeptr (x->cross) ;
		
		systhread_mutex_free (x->mymutex) ;
	}

void november_assist (t_november *x, void *b, long m, long a, char *s)
	{
		if (m == ASSIST_INLET) 
			{ 
				sprintf (s, "(bang)") ;
			} 
		else 
			{	
				sprintf (s, "How many notes") ;	
			}
	}

void november_attach (t_november *x)
	{	
		x->mypatcherview = object_attr_getobj (x->mypatcher, _sym_firstview) ;
		object_attach_byptr_register (x, x->mypatcherview, CLASS_NOBOX) ;
	}
	
void november_notify (t_november *x, t_symbol *s, t_symbol *msg, void *sender, void *data)
	{
		if (msg == _sym_attr_modified && sender == x->mypatcherview) 
			{ 			
				t_symbol *attr_name ;
				
				attr_name = (t_symbol *)object_method (data, _sym_getname) ;
				
				if (attr_name == _sym_dirty) 
					{
						x->dirty = 1 ;
					}
					
				if (attr_name == _sym_locked) 
					{
						if (patcherview_get_locked (x->mypatcherview) && x->dirty) 
							{
								pickup_matrix (x) ;
							}	
					}
				
			}
	}
	
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

void november_bang (t_november *x) 
	{		
		if (!systhread_mutex_trylock (x->mymutex))
			{
				long k = x->iter ;
				
				if (x->dirty || !x->mypatcherview)
					{
						if (!x->mypatcherview)
							{
								x->mypatcherview = object_attr_getobj (x->mypatcher, _sym_firstview) ;
								object_attach_byptr_register (x, x->mypatcherview, CLASS_NOBOX) ;
							}
						
						pickup_matrix (x) ;
					}
					
				pickup_note (x) ;
				
				while (k) {
					process_automata (x) ;
					k -- ;
					}
								
				fill_matrix (x) ;
				
				systhread_mutex_unlock (x->mymutex) ;
			}
		
		outlet_int (x->m_out, x->count) ;
	}

void november_shake (t_november *x) 
	{		
		if (!systhread_mutex_trylock (x->mymutex))
			{
				if (x->dirty || !x->mypatcherview)
					{
						if (!x->mypatcherview)
							{
								x->mypatcherview = object_attr_getobj (x->mypatcher, _sym_firstview) ;
								object_attach_byptr_register (x, x->mypatcherview, CLASS_NOBOX) ;
							}
						
						pickup_matrix (x) ;
					}
					
				pickup_note (x) ;
				
				process_shake (x) ;
				
				fill_matrix (x) ;
				
				systhread_mutex_unlock (x->mymutex) ;
			}
		
		outlet_int (x->m_out, x->count) ;
	}

void november_swap (t_november *x) 
	{		
		if (!systhread_mutex_trylock (x->mymutex))
			{
				if (x->dirty || !x->mypatcherview)
					{
						if (!x->mypatcherview)
							{
								x->mypatcherview = object_attr_getobj (x->mypatcher, _sym_firstview) ;
								object_attach_byptr_register (x, x->mypatcherview, CLASS_NOBOX) ;
							}
						
						pickup_matrix (x) ;
					}
					
				pickup_note (x) ;
			
				process_swap (x) ;
				
				fill_matrix (x) ;
				
				systhread_mutex_unlock (x->mymutex) ;
			}
		
		outlet_int (x->m_out, x->count) ;
	}
	
void november_values (t_november *x, t_symbol *s, long ac, t_atom *av) 
	{
		long i ;
		
		for (i = 0 ; i < ac ; i++)
			{
				if (atom_gettype (av + i) == A_LONG)
					{
						long k = atom_getlong (av + i) ;
						
						switch (i) {
							case 0 :	x->birth_low = CLIP (k, 0, 10) ;
										break ;
							case 1 :	x->birth_wide = CLIP (k, 0, 10) ; 
										break ;
							case 2 :	x->death_low = CLIP (k, 0, 10) ; 
										break ;
							case 3 :	x->death_wide = CLIP (k, 0, 10) ; 
										break ;
							}
					}	
			}
	} 
	
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

void pickup_matrix (t_november *x)
	{
		t_object *ptr_p = NULL ;
		
		free_badaboum_content (x) ;
		linklist_clear (x->badaboum) ;
		
		if (!(object_obex_lookup (x, gensym ("#P"), &ptr_p)))
			{
				long result = 0 ;
				
				object_method (ptr_p, _sym_iterate, (method)find_matrix, x, PI_WANTBOX | PI_DEEP, &result) ;
			}
	
		x->dirty = 0 ;			
	}	
	
long find_matrix (t_november *x, t_object *ptr_b)
	{
		t_object *ptr_p = jbox_get_patcher (ptr_b) ;
		t_symbol *sym_v = jbox_get_varname (ptr_b) ;
		t_symbol *sym_p = jpatcher_get_name (ptr_p) ;
		t_symbol *sym_b = jpatcher_get_maxclass (ptr_b) ; 
		
		if (sym_p == x->catch_name && sym_b == my_jit_matrix && sym_v == gensym (DEF_VARNAME_TO_FIND))
			{
				t_object *ptr_o		= NULL ;
				t_object *ptr_jit_o = NULL ;
				
				ptr_o = jbox_get_object (ptr_b) ;
				
				if (ptr_jit_o = max_jit_obex_jitob_get (ptr_o))
					{
						t_mat_head *mptr = NULL ;
						
						if (mptr = (t_mat_head *)sysmem_newptr (sizeof(t_mat_head)))
							{ 
								if (mptr->content = (long *)sysmem_newptr (sizeof(long) * MAX_MAT_CONTENT))
									{
										mptr->size = 0 ;
										mptr->index = 0 ;
										mptr->jptr = ptr_jit_o ;
										mptr->optr = ptr_o ;
										
										linklist_append (x->badaboum, mptr) ;	
									}
								else
									{
										sysmem_freeptr (mptr) ;
									}
							}
					}
			}

		return 0 ;
	}

void free_badaboum_content (t_november *x)
	{
		t_mat_head	*ptr1 = NULL ;
		t_mat_head	*ptr2 = NULL ;
		
		if (ptr1 = linklist_getindex (x->badaboum, 0))
			{
				while (ptr1) 
					{
						sysmem_freeptr (ptr1->content) ;
					
						linklist_next (x->badaboum, (void *)ptr1, (void *)&ptr2) ;
						ptr1 = ptr2 ;	
					}
			}
	}

void clear_badaboum_content (t_november *x)
	{
		t_mat_head	*ptr1 = NULL ;
		t_mat_head	*ptr2 = NULL ;
		
		if (ptr1 = linklist_getindex (x->badaboum, 0))
			{
				while (ptr1) 
					{
						ptr1->index = 0 ;
					
						linklist_next (x->badaboum, (void *)ptr1, (void *)&ptr2) ;
						ptr1 = ptr2 ;	
					}
			}
	}
	
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

void pickup_note (t_november *x)
	{
		t_mat_head	*ptr1 = NULL ;
		t_mat_head	*ptr2 = NULL ;
		
		if (x->step != x->temp)      
           {
				x->step = x->temp ;
				x->size_row = (MAX_POS + 1) / x->temp ;
				x->size_array = ((MAX_POS + 1) / x->temp) * 128 ;
				init_pattern (x) ;
			}
		
		clear_badaboum_content (x) ;
		
		if (ptr1 = linklist_getindex (x->badaboum, 0))
			{
				long k = 0 ;

				while (ptr1) 
					{
						t_jit_matrix_info	info ;
						long				total = 0 ;
						
						jit_object_method (ptr1->jptr, _sym_getinfo, &info) ;
						
						if (info.type == _sym_long && info.dimcount == 1 && info.planecount == MATRIX_TO_FIND_PLANECOUNT) 
							{
								long i ;
								
								for (i = 0 ; i < info.dim[0] ; i++)
									{	
										long	pos, rac ;
										t_atom	argv ;
										t_atom	*rav = NULL ;
										
										jit_atom_setlong (&argv, i) ; 
										jit_object_method (ptr1->jptr, my_getcell, my_getcell, 1, &argv, &rac, &rav) ;
										
										if (rac == MATRIX_TO_FIND_PLANECOUNT + 2 && rav)
											{
												long		w ;
												long		block = 0 ;
												t_note_head *nptr = NULL ;
												
												w = (jit_atom_getlong (rav + 3) * x->size_row) + floor (jit_atom_getlong (rav + 2) / x->step) ;
												pos = CLIP (w, 0, x->size_array - 1) ;
												
												if (x->spray)
													{
														long		*ptr3	= NULL ;
														t_note_head *nptr2  = NULL ;
														
														if (ptr3 = x->cluster)
															{
																while (*ptr3)
																	{
																		if ((pos + (*ptr3)) > 0 && (pos + (*ptr3)) < x->size_array)
																			{
																				nptr2 = x->board[(pos + (*ptr3))] ;
																				
																				if (nptr2) 
																					{
																						block = 1 ;
																					}
																			}
																					
																		ptr3 ++ ;
																	}
															}
													}
												
												if (x->board[pos])
													{
														block = 1 ;	
													}
		
												if (pos != 0 && !block && total < MAX_MAT_CONTENT && x->index < MAX_NOVEMBER_CONTENT) 
													{
														if (nptr = (t_note_head *)sysmem_newptr (sizeof(t_note_head)))
															{	
																nptr->link = k ;
																nptr->note = rav ;		
																x->board[pos] = nptr ;	
																
																ptr1->content[total] = pos ;
																x->content[x->index] = pos ;

																total++ ;
																x->index++ ;
															}
													}
											}
									}
							}
						
						ptr1->size = total ;
						ptr1->index = total ;
						
						linklist_next (x->badaboum, (void *)ptr1, (void *)&ptr2) ;
						ptr1 = ptr2 ;	
						k++ ;
					}  
			}
	}

void free_board (t_november *x)
	{
		long i ;
		
		for (i = 0 ; i < x->index ; i++)
			{
				t_note_head	*nptr = NULL ;
				long		k = x->content[i] ;
				
				if (nptr = x->board[k])
					{
						jit_freebytes (nptr->note, sizeof(t_atom) * (MATRIX_TO_FIND_PLANECOUNT + 2)) ;
						sysmem_freeptr (nptr) ;
						x->board[k] = NULL ;
					}
			}
			
		x->index = 0 ;
	}

// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

void fill_matrix (t_november *x)
	{
		long		i ;
		long		w = 0 ;
		t_mat_head	*ptr1 = NULL ;
		t_mat_head	*ptr2 = NULL ;
		
		if (ptr1 = linklist_getindex (x->badaboum, 0))
			{
				while (ptr1) 
					{
						t_atom argv, rargv ;
						
						if (ptr1->size)
							{
								jit_atom_setlong (&argv, MIN (ptr1->size, MAX_NUMBER_NOTES)) ;
								jit_object_method_typed (ptr1->jptr, _sym_dim, 1, &argv, &rargv) ;	
							}
						else
							{
								jit_atom_setlong (&argv, 1) ; 
								jit_object_method_typed (ptr1->jptr, _sym_dim, 1, &argv, &rargv) ;
								jit_object_method (ptr1->jptr, _sym_clear, _sym_clear) ;
							}
					
						linklist_next (x->badaboum, (void *)ptr1, (void *)&ptr2) ;
						ptr1 = ptr2 ;	
					}
			}
		
		for (i = 0 ; i < x->index ; i++) 
			{
				t_note_head	*nptr = NULL ;
				long		k = x->content[i] ;
		
				if (nptr = x->board[k])
					{
						t_mat_head  *mptr = NULL ;

						mptr = linklist_getindex (x->badaboum, nptr->link) ;
						mptr->size-- ;
						
						jit_atom_setlong (nptr->note, mptr->size) ;
						jit_object_method (mptr->jptr, my_setcell, my_setcell, MATRIX_TO_FIND_PLANECOUNT + 2, nptr->note) ;	
					
						w ++ ;
					}
			}
			
		if (ptr1 = linklist_getindex (x->badaboum, 0))
			{
				while (ptr1) 
					{
						jit_object_method (ptr1->optr, _sym_bang) ;
						linklist_next (x->badaboum, (void *)ptr1, (void *)&ptr2) ;
						ptr1 = ptr2 ;	
					}
			}
		
		x->count = w ;
		
		free_board (x) ;
	}
	
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

void process_automata (t_november *x)
	{
		long k ;
		long result = 0 ;
		long flag = 0 ;
		
		set_neighbourhood (x) ;
		
		while (!result && flag < SAFE_LOOP)
			{
				if (k = getrandom_content (x))
					{
						long *pat		= getrandom_pattern (x) ;
						long is_dead	= neighbourhood_death (x, k) ;
						
						if (pat)
							{
								while (*pat)
									{
										t_note_head *nptr1 = NULL ;
										
										if ((k + (*pat)) > 0 && (k + (*pat)) < x->size_array && !x->board[(k + (*pat))]) 
											{
												if (nptr1 = neighbourhood_birth (x, (k + (*pat))))
													{
														t_mat_head  *mptr  = NULL ;

														mptr = linklist_getindex (x->badaboum, nptr1->link) ;
																
														if (mptr->index < MAX_MAT_CONTENT && x->index < MAX_NOVEMBER_CONTENT)
															{
																t_note_head *nptr2 = NULL ;
																t_note_head *nptr3 = NULL ;
																
																nptr2 = (t_note_head *)sysmem_newptr (sizeof(t_note_head)) ;
																nptr3 = x->board[k] ;
																
																if (nptr2 && nptr3)
																	{
																		long		i ;
																		long		new_position ;
																		long		new_pitch ;
																		t_atom		*aptr1 = NULL ;
																		t_atom		*aptr2 = NULL ;
																		t_atom		*aptr3 = NULL ;
																																				
																		aptr1 = nptr1->note ;
																		aptr2 = jit_getbytes (sizeof(t_atom) * (MATRIX_TO_FIND_PLANECOUNT + 2)) ;
																		aptr3 = nptr3->note ;
																		
																		new_position = (((*pat) - ((round ((double)(*pat) / (double)x->size_row)) * x->size_row)) * x->step) + jit_atom_getlong (aptr3 + 2) ;
																		new_pitch = round ((double)(*pat) / (double)x->size_row) + jit_atom_getlong (aptr3 + 3) ;
																		
																		jit_atom_setlong (aptr2, jit_atom_getlong (aptr1)) ;
																		jit_atom_setsym  (aptr2 + 1, jit_atom_getsym (aptr1 + 1)) ;
																		jit_atom_setlong (aptr2 + 2, new_position) ;  
																		jit_atom_setlong (aptr2 + 3, new_pitch) ;
																		
																		for (i = 4 ; i < (MATRIX_TO_FIND_PLANECOUNT + 2) ; i++)
																			{	
																				jit_atom_setlong (aptr2 + i, jit_atom_getlong (aptr1 + i)) ;
																			}
																			
																		nptr2->link = nptr1->link ;
																		nptr2->note = aptr2 ;
																
																		x->board[(k + (*pat))] = nptr2 ;
																		x->content[x->index] = (k + (*pat)) ;
																		x->index++ ;
																		
																		mptr->content[mptr->index] = (k + (*pat)) ;																																				
																		mptr->index++ ;
																		mptr->size++ ;
																	}
																else if (nptr2)
																	{
																		sysmem_freeptr (nptr2) ;
																	}
															}
														
														result = 1 ;
														break ;
													}
											}
											
										pat ++ ;
									}
							}
							
						if (is_dead)
							{
								t_note_head	*nptr = NULL ;
								
								if (nptr = x->board[k])
									{
										t_mat_head	*mptr = NULL ;
										
										jit_freebytes (nptr->note, sizeof(t_atom) * (MATRIX_TO_FIND_PLANECOUNT + 2)) ;
										
										mptr = linklist_getindex (x->badaboum, nptr->link) ;
										mptr->size-- ;
											
										sysmem_freeptr (nptr) ;
										x->board[k] = NULL  ;
									}
								
								result = 1 ;
							}
						
						flag ++ ;
					}
				else
					{
						result = 1 ;
					}
			}
	}

long neighbourhood_death (t_november *x, long n)
	{
		long		*ptr	= NULL ;
		t_note_head	*nptr	= NULL ;
		long		k = 0 ;
		
		if (ptr = x->neighbourhood)
			{
				while (*ptr)
					{
						if ((n + (*ptr)) > 0 && (n + (*ptr)) < x->size_array)
							{
								if (nptr = x->board[(n + (*ptr))])
									{
										k ++ ;
									}
							}
							
						ptr ++ ;
					}
			}
		
		if (k < x->death_low || k > (x->death_low + x->death_wide))
			{
				return 1 ;
			}
		else
			{
				return 0 ;
			}	
	}

t_note_head *neighbourhood_birth (t_november *x, long n)
	{
		long		k = 0 ;
		long		flag = 0 ;
		long		*ptr1	= NULL ;
		t_note_head	*nptr1	= NULL ;
		t_note_head *rptr	= NULL ;
		t_note_head *tab[MAX_SIZE_PATTERN] ;
		
		if (ptr1 = x->neighbourhood)
			{
				while (*ptr1)
					{
						if ((n + (*ptr1)) > 0 && (n + (*ptr1)) < x->size_array)
							{
								nptr1 = x->board[(n + (*ptr1))] ;
								
								if (nptr1) 
									{
										tab[k] = nptr1 ;
										k ++ ;
									}
							}
									
						ptr1 ++ ;
					}
			}
				
		if (x->spray)
			{
				long		*ptr2	= NULL ;
				t_note_head *nptr2  = NULL ;
				
				if (ptr2 = x->cluster)
					{
						while (*ptr2)
							{
								if ((n + (*ptr2)) > 0 && (n + (*ptr2)) < x->size_array)
									{
										nptr2 = x->board[(n + (*ptr2))] ;
										
										if (nptr2) 
											{
												flag = 1 ;
											}
									}
											
								ptr2 ++ ;
							}
					}
			}
				
		if (k >= x->birth_low && k <= (x->birth_low + x->birth_wide) && k != 0)
			{
				if (flag == 0)
					{						
						rptr = tab[(rand () % k)] ;	
					}
			}
		
		return rptr ;
	}
	
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

void process_shake (t_november *x)
	{
		t_linklist	*temp = NULL ;
		
		if (temp = linklist_new ( )) 
			{
				long i ;

				linklist_flags (temp, OBJ_FLAG_REF) ;
				
				for (i = 0 ; i < x->index ; i++) 
					{
						long		k = x->content[i] ;
						t_note_head	*nptr = NULL ;
				
						if (nptr = x->board[k])
							{
								linklist_append (temp, (void *)nptr->link) ;
							}
					}
				
				linklist_shuffle (temp) ;
				
				for (i = 0 ; i < x->index ; i++) 
					{
						long		k = x->content[i] ;
						t_note_head	*nptr = NULL ;
				
						if (nptr = x->board[k])
							{
								nptr->link = (long)linklist_getindex (temp, i) ;
							}
					}
				
				object_free (temp) ;
			}
	}

void process_swap (t_november *x)
	{
		t_linklist	*temp = NULL ;
		
		if (temp = linklist_new ( )) 
			{
				t_mat_head	*ptr1 = NULL ;
				t_mat_head	*ptr2 = NULL ;

				linklist_flags (temp, OBJ_FLAG_REF) ;
				
				if (ptr1 = linklist_getindex (x->badaboum, 0))
					{
						while (ptr1) 
							{
								linklist_append (temp, (void *)ptr1->size) ;
							
								linklist_next (x->badaboum, (void *)ptr1, (void *)&ptr2) ;
								ptr1 = ptr2 ;	
							}
					}
				
				if (linklist_getsize (x->badaboum) == 2)
					{
						linklist_reverse (x->badaboum) ;
					}
				else
					{
						linklist_shuffle (x->badaboum) ;
					}
				
				if (ptr1 = linklist_getindex (x->badaboum, 0))
					{
						long i = 0 ;
						
						while (ptr1) 
							{
								ptr1->size = (long)linklist_getindex (temp, i) ;
							
								linklist_next (x->badaboum, (void *)ptr1, (void *)&ptr2) ;
								ptr1 = ptr2 ;	
								i ++ ;
							}
					}
				
				object_free (temp) ;
			}
	}

// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

void init_pattern (t_november *x)
	{
		if (x->Hpat_A)
			{ sysmem_freeptr (x->Hpat_A) ; }
		if (x->Hpat_B)
			{ sysmem_freeptr (x->Hpat_B) ; }
		if (x->Hpat_C)
			{ sysmem_freeptr (x->Hpat_C) ; }
		if (x->Hpat_D)
			{ sysmem_freeptr (x->Hpat_D) ; }
			
		if (x->cluster)
			{ sysmem_freeptr (x->cluster) ; }
			
		if (x->moore)
			{ sysmem_freeptr (x->moore) ; }
		if (x->diamond)
			{ sysmem_freeptr (x->diamond) ; }
		if (x->square_head)
			{ sysmem_freeptr (x->square_head) ; }
		if (x->rectangle)
			{ sysmem_freeptr (x->rectangle) ; }
		if (x->cross)
			{ sysmem_freeptr (x->cross) ; }
		
		if (x->Hpat_A = (long *)sysmem_newptr (sizeof(long) * 7))
			{
				x->Hpat_A[0] = ALFA ;
				x->Hpat_A[1] = CHARLIE ;
				x->Hpat_A[2] = DELTA ;
				x->Hpat_A[3] = ECHO ;
				x->Hpat_A[4] = FOXTROT ;
				x->Hpat_A[5] = HOTEL ;
				x->Hpat_A[6] = 0 ;
			}
		
		if (x->Hpat_B = (long *)sysmem_newptr (sizeof(long) * 7))
			{
				x->Hpat_B[0] = HOTEL ;
				x->Hpat_B[1] = FOXTROT ;
				x->Hpat_B[2] = ECHO ;
				x->Hpat_B[3] = DELTA ;
				x->Hpat_B[4] = CHARLIE ;
				x->Hpat_B[5] = ALFA ;
				x->Hpat_B[6] = 0 ;
			}
		
		if (x->Hpat_C = (long *)sysmem_newptr (sizeof(long) * 7))
			{
				x->Hpat_C[0] = ALFA ;
				x->Hpat_C[1] = DELTA ;
				x->Hpat_C[2] = FOXTROT ;
				x->Hpat_C[3] = CHARLIE ;
				x->Hpat_C[4] = ECHO ;
				x->Hpat_C[5] = HOTEL ;
				x->Hpat_C[6] = 0 ;
			}
		
		if (x->Hpat_D = (long *)sysmem_newptr (sizeof(long) * 7))
			{
				x->Hpat_D[0] = HOTEL ;
				x->Hpat_D[1] = ECHO ;
				x->Hpat_D[2] = CHARLIE ;
				x->Hpat_D[3] = FOXTROT ;
				x->Hpat_D[4] = DELTA ;
				x->Hpat_D[5] = ALFA ;
				x->Hpat_D[6] = 0 ;	
			}
		
		if (x->cluster = (long *)sysmem_newptr (sizeof(long) * 3))
			{
				x->cluster[0] = BRAVO ;
				x->cluster[1] = GULF ;
				x->cluster[2] = 0 ;
			}
		
		if (x->moore = (long *)sysmem_newptr (sizeof(long) * 9))  
			{
				x->moore[0] = ALFA ;
				x->moore[1] = BRAVO ;
				x->moore[2] = CHARLIE ;
				x->moore[3] = DELTA ;
				x->moore[4] = ECHO ;
				x->moore[5] = FOXTROT ;
				x->moore[6] = GULF ;
				x->moore[7] = HOTEL ;
				x->moore[8] = 0 ;
			}
			
		if (x->diamond = (long *)sysmem_newptr (sizeof(long) * 13))  
			{
				x->diamond[0] = WHISKEY ;
				x->diamond[1] = ALFA ;
				x->diamond[2] = BRAVO ;
				x->diamond[3] = CHARLIE ;
				x->diamond[4] = XRAY ;
				x->diamond[5] = DELTA ;
				x->diamond[6] = ECHO ;
				x->diamond[7] = YANKEE ;
				x->diamond[8] = FOXTROT ;
				x->diamond[9] = GULF ;
				x->diamond[10] = HOTEL ;
				x->diamond[11] = ZULU ;
				x->diamond[12] = 0 ;
			}
		
		if (x->square_head = (long *)sysmem_newptr (sizeof(long) * 17))  
			{
				x->square_head[0] = MIKE ;
				x->square_head[1] = WHISKEY ;
				x->square_head[2] = NOVEMBER ;
				x->square_head[3] = ALFA ;
				x->square_head[4] = BRAVO ;
				x->square_head[5] = CHARLIE ;
				x->square_head[6] = XRAY ;
				x->square_head[7] = DELTA ;
				x->square_head[8] = ECHO ;
				x->square_head[9] = YANKEE ;
				x->square_head[10] = FOXTROT ;
				x->square_head[11] = GULF ;
				x->square_head[12] = HOTEL ;
				x->square_head[13] = OSCAR ;
				x->square_head[14] = ZULU ;
				x->square_head[15] = PAPA ;
				x->square_head[16] = 0 ;
			}
		
		if (x->rectangle = (long *)sysmem_newptr (sizeof(long) * 15))  
			{
				x->rectangle[0] = MIKE ;
				x->rectangle[1] = WHISKEY ;
				x->rectangle[2] = NOVEMBER ;
				x->rectangle[3] = ALFA ;
				x->rectangle[4] = BRAVO ;
				x->rectangle[5] = CHARLIE ;
				x->rectangle[6] = DELTA ;
				x->rectangle[7] = ECHO ;
				x->rectangle[8] = FOXTROT ;
				x->rectangle[9] = GULF ;
				x->rectangle[10] = HOTEL ;
				x->rectangle[11] = OSCAR ;
				x->rectangle[12] = ZULU ;
				x->rectangle[13] = PAPA ;
				x->rectangle[14] = 0 ;
			}
		
		if (x->cross = (long *)sysmem_newptr (sizeof(long) * 9))
			{
				x->cross[0] = INDIA ;
				x->cross[1] = JULIET ;
				x->cross[2] = ALFA ;
				x->cross[3] = CHARLIE ;
				x->cross[4] = FOXTROT ;
				x->cross[5] = HOTEL ;
				x->cross[6] = KILO ;
				x->cross[7] = LIMA ;
				x->cross[8] = 0 ;
			}
	}

void set_neighbourhood (t_november *x)
	{
		x->neighbourhood = NULL ;
		
		switch (x->enumindex) {
			case 0	:	x->neighbourhood = x->moore ;
						break ;
			case 1	:	x->neighbourhood = x->diamond ;
						break ;
			case 2	:	x->neighbourhood = x->square_head ; 
						break ;
			case 3	:	x->neighbourhood = x->rectangle ; 
						break ;
			case 4	:	x->neighbourhood = x->cross ; 
						break ;
			}
	}

long *getrandom_pattern (t_november *x)
	{
		long *ptr = NULL ;
		long k = rand () % H_PATTERN_NUMBER ;
		
		switch (k) {
			case 0		:	ptr = x->Hpat_A ;
							break ;
			case 1		:	ptr = x->Hpat_B ;
							break ;
			case 2		:	ptr = x->Hpat_C ;
							break ;
			case 3		:	ptr = x->Hpat_D ;
							break ;
			}
		
		return ptr ;
	}

// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------

long getrandom_content (t_november *x)
	{
		long		k = 0 ;
		long		h, m ;
		t_mat_head	*ptr ;
		
		if (m = linklist_getsize (x->badaboum))
			{
				h = rand () % m ;
				ptr = linklist_getindex (x->badaboum, h) ;
				
				if (ptr->size) 
					{
						long w, i ;
						long n = ptr->index ;
						
						h = rand () % n ;
						
						for (i = 0 ; i < n ; i++)
							{
								w = ptr->content[h] ;
								
								if (x->board[w])
									{ 
										k = w ;
										break ; 
									}
								else
									{ 
										h++ ; 
									}
									
								if (h = n)
									{ 
										h = 0 ; 
									}
							}
					}
			}
				
		return k ;
	}
	
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------:x