/*
 ol.mousewheel v 0.9 by oli larkin
 www.olilarkin.co.uk
 
 monitor mousewheel events in a window (mac/pc)
 arg 1 specifies a named jitter context. If not present it takes the window that the object is in
 
 
 */

#include "ext.h"  
#include "jit.common.h"

typedef struct _olmousewheel	
{
	t_object p_ob;		
	void *p_vout;
	void *p_hout;
	void *p_modout;
	
	t_symbol	*ctxt; // jit.window ctxt if specified
	
#ifdef WIN_VERSION
	HWND hWnd;
	WNDPROC oldproc;
#else

	WindowRef winRef;
	EventHandlerRef eventHandlerRef;
#endif
} t_olmousewheel;

void *olmousewheel_class;
void *olmousewheel_new(t_symbol *jitctxt);
void olmousewheel_free(t_olmousewheel *x);
void olmousewheel_start(t_olmousewheel *x);
void olmousewheel_stop(t_olmousewheel *x);
void olmousewheel_int(t_olmousewheel *x, long n);
void olmousewheel_assist(t_olmousewheel *x, void *b, long m, long a, char *s);

#ifdef WIN_VERSION
LRESULT CALLBACK MyOwnWndProc(HWND hwnd, UINT umsg, WPARAM wParam, LPARAM lParam);
#else
OSStatus handleWindowEvent (EventHandlerCallRef callRef, EventRef theEvent, t_olmousewheel *x);
#endif

int main(void)
{
	setup((t_messlist **)&olmousewheel_class, (method)olmousewheel_new, (method)olmousewheel_free, (short)sizeof(t_olmousewheel), 0L, A_DEFSYM, 0);
	
	addmess((method)olmousewheel_start, "start", 0);
	addmess((method)olmousewheel_stop, "stop", 0); 
	addint((method)olmousewheel_int);
	addmess((method)olmousewheel_assist,"assist",A_CANT,0);		
	return 0;
}

void *olmousewheel_new(t_symbol *jitctxt)		
{
	t_olmousewheel *x;	
	
	x = (t_olmousewheel *)newobject(olmousewheel_class); 
	x->ctxt = jitctxt;
	

	x->p_modout = intout(x);
	x->p_hout = intout(x);
	x->p_vout = intout(x);
	
	return(x);					
}

void olmousewheel_free(t_olmousewheel *x)
{
	olmousewheel_stop(x);
}

#ifdef WIN_VERSION
void olmousewheel_start(t_olmousewheel *x)
{
	
	void *window;
	
	olmousewheel_stop(x);
	
	if(x->oldproc == NULL)
	{
		window = jit_object_findregistered(x->ctxt);
		if(window) jit_object_method(window, gensym("get_window_ptr"), &x->hWnd);
		else x->hWnd = main_get_client(); //if no argument, get the patcher window 
		
		if(SetProp(x->hWnd,"olmousewheel", x)) 
			x->oldproc = (WNDPROC) SetWindowLong(x->hWnd, GWL_WNDPROC,(LONG)MyOwnWndProc);
		else post("Couldn't setProp()");
	}
}

LRESULT CALLBACK MyOwnWndProc(HWND hwnd, UINT umsg, WPARAM wParam, LPARAM lParam)
{
	t_olmousewheel *x = (t_olmousewheel*) GetProp(hwnd,"olmousewheel");
	
	switch (umsg)
	{
		case WM_MOUSEWHEEL:
			outlet_int(x->p_vout, GET_WHEEL_DELTA_WPARAM(wParam)/120);
			outlet_int(x->p_modout, GET_KEYSTATE_WPARAM(wParam));
			
			return 0; // return, or can also pass to orig proc
		case WM_MOUSEHWHEEL:
			outlet_int(x->p_hout, GET_WHEEL_DELTA_WPARAM(wParam)/120);
			outlet_int(x->p_modout, GET_KEYSTATE_WPARAM(wParam));

			return 0; // return, or can also pass to orig proc
//		default:
//					   return 0;
	}
	
	if (x && x->oldproc) return CallWindowProc(x->oldproc, hwnd, umsg, wParam, lParam);
	else return DefMDIChildProc(hwnd, umsg, wParam, lParam);
} 

void olmousewheel_stop(t_olmousewheel *x)
{
	if(x->oldproc)
	{
		SetWindowLong(x->hWnd, GWL_WNDPROC, x->oldproc);
		x->oldproc = 0;
	}
	RemoveProp(x->hWnd, "olmousewheel");
}

#else

void olmousewheel_start(t_olmousewheel *x)
{
	void *window;
	
	olmousewheel_stop(x);
	
	window = jit_object_findregistered(x->ctxt);
	if(window) jit_object_method(window, gensym("get_window_ptr"), &x->winRef);
	else x->winRef = FrontWindow(); //if no argument, get the patcher window 
	
	x->eventHandlerRef = 0;
	
	const EventTypeSpec eventTypes[] =
	{
		{ kEventClassMouse, kEventMouseWheelMoved },
	};
	
	static EventHandlerUPP handleWindowEventUPP = 0;
	
	if (handleWindowEventUPP == 0) handleWindowEventUPP = NewEventHandlerUPP (handleWindowEvent);
	
	InstallWindowEventHandler (x->winRef, handleWindowEventUPP,
							   GetEventTypeCount (eventTypes), eventTypes,
							   (t_olmousewheel *) x, &x->eventHandlerRef);
}

void olmousewheel_stop(t_olmousewheel *x)
{
	if (x->eventHandlerRef != 0)
	{
		RemoveEventHandler (x->eventHandlerRef);
		x->eventHandlerRef = 0;
	}
}

OSStatus handleWindowEvent (EventHandlerCallRef callRef, EventRef theEvent, t_olmousewheel *x)
{
	if ( GetEventClass (theEvent) == kEventClassMouse)
	{
		if (GetEventKind (theEvent) == kEventMouseWheelMoved)
		{
			
			OSStatus status;
            UInt32 modifiers;
            EventMouseWheelAxis axis;
            SInt32 delta;
            
            status = GetEventParameter( theEvent, kEventParamKeyModifiers, typeUInt32, 
									   NULL, sizeof(modifiers), NULL, &modifiers );
            //ASSERT_NO_ERR(status);
			
            status = GetEventParameter( theEvent, kEventParamMouseWheelAxis, 
									   typeMouseWheelAxis, NULL, sizeof(axis), NULL, &axis );
            //ASSERT_NO_ERR(status);
			
            status = GetEventParameter( theEvent, kEventParamMouseWheelDelta, 
									   typeLongInteger, NULL, sizeof(delta), NULL, &delta );
            //ASSERT_NO_ERR(status);
			
            if ( axis == kEventMouseWheelAxisY )
            {
				outlet_int(x->p_modout, modifiers);
                outlet_int(x->p_hout, delta);
				return noErr;
            }
			else
			{
				outlet_int(x->p_modout, modifiers);
				outlet_int(x->p_vout, delta);
				return noErr;
			}
			
		}
		else return eventNotHandledErr;
	}
	else return eventNotHandledErr;
}
#endif

void olmousewheel_int(t_olmousewheel *x, long n)
{
	switch(n)
	{
		case 0:
			olmousewheel_stop(x);
			break;
		case 1:
			olmousewheel_start(x);
			break;
		default:
			post("%s","int only, 0 or 1");
			break;
	}
	
}

void olmousewheel_assist(t_olmousewheel *x, void *b, long m, long a, char *s)
{
	
	if (m == ASSIST_INLET) sprintf(s,"Start / 1 = Attach handler to patcher window, Stop / 0 = Detach");
	else
	{
		switch(a)
		{
			case 0:
				sprintf(s,"Mousewheel X Delta");
				break;
			case 1:
				sprintf(s,"Mousewheel Y Delta");
				break;
			case 2:
				sprintf(s,"Modifiers");
				break;
		}
	}
}